package logic;

import java.io.*;
import java.util.*;

/**
 * Class to create a random book with random author and other features
 * @author Jens
 *
 */
public class Book implements Serializable{
	private static final long serialVersionUID = 1L;
	private final String title, language;
	private final Topic topic;
	private String titles = "Fibel,Handbuch,Grundbegriffe,Die Kunst,Von den Gefahren,Vom Wesen,Geheimnisse,Das Buch,Almanach,Lexikon,Chronik,Kompendium,Enzyklopdie,Kodex,Grundlagen,Theorie";
	private int complexity, count, cost;
	private final long id;
	private int bookyear;
	private final Author author;
	private ArrayList<Spell> spells;
	private ArrayList<String> notices;
	private ArrayList<Special> specials;
	
	/**
	 * Constructs a new book with random specials, spells (if any), author
	 * and so on
	 * @param id for the BookCollection because title is a poor identifier
	 */
	public Book(long id){
		this.id=id;
		this.notices = new ArrayList<String>();
		String[] booktitles = this.titles.split(",");
		this.complexity = new Random().nextInt(6)+
		new Random().nextInt(6)+new Random().nextInt(6);
		this.title = booktitles[complexity];
		this.complexity = this.complexity+3;
		this.topic = new Topic();
		this.count = new Random().nextInt(20)-9;
		if(this.count<1){
			this.count=1;
		}
		this.cost = new Random().nextInt(20)+
		new Random().nextInt(20)+new Random().nextInt(20)+3;
		int spellcount = new Random().nextInt(6)-3;
		if(spellcount<0){
			spellcount=0;
		}
		if(spellcount>0){
			this.spells = SpellCollection.getInstance().getSpellsByToken(spellcount, this.topic.getTokens());	
		}
		else{
			this.spells = new ArrayList<Spell>();
		}
		this.author = new Author();
		try {
			this.bookyear = this.author.getDeathyear()-(new Random().nextInt(30)+2);
		} catch (HeIsNotDeadException e) {
			this.bookyear = 1080-this.author.getAge()+new Random().nextInt(20);
		}
		this.specials = SpecialCollection.getInstance().getSpecialsByTag(new Random().nextInt(5)+1, this.topic.getTokens());
		this.language = getTheLanguage(this.author.getRegion(), this.bookyear, this.author.getAlign());
	}
	
	private String getTheLanguage(String region, int bookyear2, String align) {
		if (align.equals("Schwarzmagier")||align.equals("Schwarzmagierin")){
			if (new Random().nextInt() % 3 == 0){
				return "Zhayad";
			}
		}
		if (region.equals("Mittelreich")){
			if(bookyear2<500){
				return "Imperiale Zeichen und Aureliani";
			}
			else{
				return "Bosparano";
			}
		}
		if (region.equals("Horasreich")){
			if(bookyear2<500){
				return "Imperiale Zeichen und Aureliani";
			}
			else{
				return "Bosparano";
			}
		}
		if (region.equals("Bornland")){
			if(bookyear2<500){
				return "Altes Alaani";
			}
			else{
				return "Bosparano";
			}
		}
		if (region.equals("Sueden")){
			if(bookyear2<500){
				return "Imperiale Zeichen und Aureliani";
			}
			else{
				return "Bosparano";
			}
		}
		if (region.equals("Thorwal")){
			if(bookyear2<500){
				return "Hjaldingische Runen";
			}
			else{
				return "Bosparano";
			}
		}
		if (region.equals("Tulamiden")){
			if(bookyear2<500){
				return "Chrmk";
			}
			else{
				return "Ur-Tulamidya";
			}
		}
		else{
			return "Nanduria";
		}
	}

	/**
	 * Standard short String with title, cost and year
	 */
	public String toString(){
		return this.title+" "+this.topic.getMainTopic()+" ("+this.spells.size()
		+" Thesen, Wert: "+this.cost+" Dukaten aus dem Jahr "+this.bookyear+" BF)";
	}
	
	/**
	 * The toString-method for textfileoutput with notices
	 * Interfaces
	 */
	public String toLongString(){
		return "======================================\n"+
		this.title+" "+this.topic.getMainTopic()+"\ngeschrieben von "+
		this.author.toString()+"\n"+this.getEpoche()+
		this.getLanguage()+
		this.spellcounter()+this.counter()+this.costFormat()+
		this.printSpecials()+this.printNotices();
	}
	
	/**
	 * The toString-method for view in GUI without notices
	 * @return
	 */
	public String toLongStringWithoutNotices() {
		return "======================================\n"+
		this.title+" "+this.topic.getMainTopic()+"\ngeschrieben von "+
		this.author.toString()+"\n"+this.getEpoche()+
		this.getLanguage()+
		this.spellcounter()+this.counter()+this.costFormat()
		+this.printSpecials();
	}

	private String printSpecials() {
		String specials = "Spezielle Erfahrungen: ";
		for(Special s : this.specials){
			specials += s.toString()+", ";
		}
		specials = specials.substring(0, specials.length()-2);
		return specials+"\n";
	}

	private String getLanguage() {
		return "\nErfordert "+this.language+" "+this.complexity;
	}

	private String printNotices() {
		String notices = "";
		if (this.notices.size()>0){
			notices += "===\nNotizen:\n";
			for (String singlenotice : this.notices){
				notices += singlenotice+"\n";
			}
		}
		return notices;
	}

	/**
	 * Get the epoche in which the book was written
	 * @return
	 */
	private String getEpoche() {
		if(this.bookyear<0){
			return "Dieses Buch wurde kurz vor Bosparans Fall geschrieben.";
		}
		if(0<= this.bookyear && this.bookyear <= 50){
			return "Dieses Buch wurde direkt nach Bosparans Fall geschrieben.";
		}
		if(51<= this.bookyear && this.bookyear <= 280){
			return "Dieses Buch entstand zur Zeit der klugen Kaiser.";
		}
		if(281<= this.bookyear && this.bookyear <= 410){
			return "Dieses Buch wurde zur Zeit der Priesterkaiser geschrieben.";
		}
		if(411<= this.bookyear && this.bookyear <= 534){
			return "Dieses Buch entstand unter der Regentschaft Rohals.";
		}
		if(535<= this.bookyear && this.bookyear <= 601){
			return "Dieses Buch wurde in den Magierkriegen verfasst.";
		}
		if(602<= this.bookyear && this.bookyear <= 685){
			return "Dieses Buch entstand kurz nach den Magierkriegen.";
		}
		if(686<= this.bookyear && this.bookyear <= 700){
			return "Dieses Buch wurde geschrieben, als sich die Al'Anfaner Boronkriche bildete";
		}
		if(701<= this.bookyear && this.bookyear <= 901){
			return "Dieses Buch entstand in fast ereignislosen Zeiten!";
		}
		if(902<= this.bookyear && this.bookyear <= 932){
			return "Dieses Buch wurde in den kaiserlosen Zeiten geschrieben!";
		}
		if(933<= this.bookyear && this.bookyear <= 947){
			return "Dieses Buch wurde geschrieben, als Kaiser Perval den Garether Thron an sich riss.";
		}
		if(948<= this.bookyear && this.bookyear <= 974){
			return "Dieses Buch entstand, als Bardo und Cella sich ihren kaiserlichen Gelsten hingaben.";
		}
		if(975<= this.bookyear && this.bookyear <= 996){
			return "Dieses Buch wurde geschrieben, als Kaiser Reto die malosen Zwillinge Bardo und Cella von Kaiserthron stie.";
		}
		if(this.bookyear == 997){
			return "Dieses Buch entstand in den Jahr, als sich Kaiser Hal zum Gott krnen lie.";
		}
		if(998<= this.bookyear && this.bookyear <= 1010){
			return "Dieses Buch entstand whrend der Regierungszeit des Gottkaisers Hal.";
		}
		if(1011<= this.bookyear && this.bookyear <= 1018){
			return "Dieses Buch entstand in der Zeit vor Borbarads Invasion.";
		}
		if(1019<= this.bookyear && this.bookyear <= 1021){
			return "Dieses Buch entstand whrend Borbarads Invasion.";
		}
		if(1021<= this.bookyear && this.bookyear <= 1026){
			return "Dieses Buch entstand kurz nach dem Borbaradkrieg.";
		}
		if(1027<= this.bookyear && this.bookyear <= 1028){
			return "Dieses Buch entstand whrend des Jahr des Feuers.";
		}
		if(1029<= this.bookyear){
			return "Dieses Buch entstand nach dem Jahr des Feuers.";
		}
		else return "";
	}

	/**
	 * output format in correct german syntax
	 * @return
	 */
	private String costFormat(){
		if(this.count==1){
			return "im Wert von "+this.cost+" Dukaten\n";
		}
		else{
			return "im Wert von jeweils "+this.cost+" Dukaten\n";
		}
	}
	
	/**
	 * Nicer format for Thesiscount-Output
	 * @return
	 */
	private String spellcounter(){
		if(this.spells.size()==0){
			return " und beinhaltet keine Thesen.\n";
		}
		else if(this.spells.size()==1){
			return " und beinhaltet eine These: "+this.spells.get(0).toString();
		}
		else{
			return " und beinhaltet "+this.spells.size()+" Thesen:\n"+this.listAllSpells();
		}
	}
	
	private String listAllSpells() {
		String allSpells = "";
		for(Spell spell: this.spells){
			allSpells = allSpells+spell.toString();
		}
		return allSpells;
	}

	/**
	 * Nicer format for output of the total amount of books of this type
	 * @return
	 */
	private String counter(){
		if(this.count==1){
			return "Es existiert ein Exemplar ";
		}
		else{
			return "Es existieren "+this.count+" Exemplare ";
		}
	}

	public long getId() {
		return id;
	}
	
	/**
	 * adds a notice
	 * @param notice String
	 */
	public void addNotice(String notice){
		this.notices.add(notice);
	}
	
	/**
	 * get all Notices
	 * @return ArrayList
	 */
	public ArrayList<String> getNotices(){
		return this.notices;
	}
	
	/**
	 * delete one notice from the book
	 * @param notice the notice to remove
	 * @return true when successfully removed
	 */
	public boolean removeNotice(String notice){
		boolean found=false;
		if(this.notices.remove(notice)){
			found=true;
		}
		return found;
	}

	/**
	 * show the title - it's not the whole title, just the beginning
	 * @return
	 */
	public String getTitle() {
		return title;
	}

	/**
	 * gets the second part of the book's title
	 * @return
	 */
	public Topic getTopic() {
		return topic;
	}
	
}
